/*
 * Copyright 2019 Contributors to the Parsec project.
 * SPDX-License-Identifier: Apache-2.0
 */
syntax = "proto3";

package list_opcodes;

message Operation { uint32 provider_id = 1; } // Cast down to 8 bits

message Result { repeated uint32 opcodes = 1; }
