/*
 * Copyright 2020 Contributors to the Parsec project.
 * SPDX-License-Identifier: Apache-2.0
 */
syntax = "proto3";

package psa_cipher_encrypt;

import "psa_algorithm.proto";

message Operation {
  string key_name = 1;
  psa_algorithm.Algorithm.Cipher alg = 2;
  bytes plaintext = 3;
}

message Result { bytes ciphertext = 1; }