/*
 * Copyright 2019 Contributors to the Parsec project.
 * SPDX-License-Identifier: Apache-2.0
 */
syntax = "proto3";

package ping;

message Operation {}

message Result {
  uint32 wire_protocol_version_maj = 1; // Cast down to 8 bits
  uint32 wire_protocol_version_min = 2; // Cast down to 8 bits
}
