/*
 * Copyright 2020 Contributors to the Parsec project.
 * SPDX-License-Identifier: Apache-2.0
 */
syntax = "proto3";

package psa_hash_compute;

import "psa_algorithm.proto";

message Operation {
  psa_algorithm.Algorithm.Hash alg = 1;
  bytes input = 2;
}

message Result { bytes hash = 1; }
