/*
 * Copyright 2020 Contributors to the Parsec project.
 * SPDX-License-Identifier: Apache-2.0
 */
syntax = "proto3";

package psa_generate_random;

message Operation { uint64 size = 1; }

message Result { bytes random_bytes = 1; }
