/*
 * Copyright 2021 Contributors to the Parsec project.
 * SPDX-License-Identifier: Apache-2.0
 */
syntax = "proto3";

package list_clients;

message Operation {}

message Result { repeated string clients = 1; }
