/*
 * Copyright 2020 Contributors to the Parsec project.
 * SPDX-License-Identifier: Apache-2.0
 */
syntax = "proto3";

package psa_export_key;

message Operation { string key_name = 1; }

message Result { bytes data = 1; }
