/*
 * Copyright 2019 Contributors to the Parsec project.
 * SPDX-License-Identifier: Apache-2.0
 */
syntax = "proto3";

package list_providers;

message ProviderInfo {
  string uuid = 1;
  string description = 2;
  string vendor = 3;
  uint32 version_maj = 4;
  uint32 version_min = 5;
  uint32 version_rev = 6;
  uint32 id = 7;
}

message Operation {}

message Result { repeated ProviderInfo providers = 1; }
