/*
 * Copyright 2020 Contributors to the Parsec project.
 * SPDX-License-Identifier: Apache-2.0
 */
syntax = "proto3";

package list_authenticators;

message AuthenticatorInfo {
  string description = 1;
  uint32 version_maj = 2;
  uint32 version_min = 3;
  uint32 version_rev = 4;
  uint32 id = 5;
}

message Operation {}

message Result { repeated AuthenticatorInfo authenticators = 1; }
