/*
 * Copyright 2021 Contributors to the Parsec project.
 * SPDX-License-Identifier: Apache-2.0
 */
syntax = "proto3";

package delete_client;

message Operation { string client = 1; }

message Result {}
