/*
 * Copyright 2019 Contributors to the Parsec project.
 * SPDX-License-Identifier: Apache-2.0
 */
syntax = "proto3";

package psa_generate_key;

import "psa_key_attributes.proto";

message Operation {
  string key_name = 1;
  psa_key_attributes.KeyAttributes attributes = 2;
}

message Result {}
