/*
 * Copyright 2020 Contributors to the Parsec project.
 * SPDX-License-Identifier: Apache-2.0
 */
syntax = "proto3";

package psa_asymmetric_decrypt;

import "psa_algorithm.proto";

message Operation {
  string key_name = 1;
  psa_algorithm.Algorithm.AsymmetricEncryption alg = 2;
  bytes ciphertext = 3;
  bytes salt = 4;
}

message Result { bytes plaintext = 1; }